"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = fastRenderToStaticMarkup;
exports.h = void 0;

var _react = _interopRequireDefault(require("react"));

var _utils = require("./utils");

var _consts = require("./consts");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var isValidElement = function isValidElement(element) {
  return element && element.hasOwnProperty('html');
};

var cloneElement = function cloneElement(element) {
  if (isValidElement(element)) {
    return element.html;
  } else {
    return '';
  }
};

var reactRuntime = {
  oldH: _react.default.createElement,
  oldIsValidElement: _react.default.isValidElement,
  oldCloneElement: _react.default.cloneElement,
  mock: function mock() {
    _react.default.createElement = h;
    _react.default.isValidElement = isValidElement;
    _react.default.cloneElement = cloneElement;
  },
  recovery: function recovery() {
    _react.default.createElement = this.oldH;
    _react.default.isValidElement = this.oldIsValidElement;
    _react.default.cloneElement = this.cloneElement;
  }
};

var convertStyleAttr = function convertStyleAttr(value) {
  var styleStr = '';
  var delimiter = '';

  for (var cssProperty in value) {
    var cssPValue = value[cssProperty];

    if (typeof cssPValue === 'number' && cssPValue !== 0 && !_consts.unitLessNumber[cssProperty]) {
      cssPValue += 'px';
    }

    styleStr += delimiter + (0, _utils.camelToKebab)(cssProperty) + ':' + cssPValue;
    delimiter = ';';
  }

  return styleStr;
};

var joinDomAttr = function joinDomAttr(key, value) {
  return ' ' + key + '="' + value + '"';
};

var getStaticMarkupAttrStr = function getStaticMarkupAttrStr(attrs) {
  var attrStr = '';

  for (var key in attrs) {
    if (_consts.filterAttrs[key]) {
      continue;
    }

    var domAttr = _consts.domAttrs[key];
    var data = attrs[key];

    if (key === 'style' && data) {
      // 这里不使用typeof xxx === 'object'，babel会将其转化为_typeof函数
      var value = typeof data === 'string' ? data : convertStyleAttr(data);
      attrStr += joinDomAttr(key, value);
      continue;
    }

    if (domAttr) {
      // dom attr
      if ((0, _utils.validateValueForDomAttr)(data)) {
        attrStr += joinDomAttr(domAttr, (0, _utils.escape)(data));
      }
    } else {
      // 自定义attr
      if ((0, _utils.validateValueForSelfDefineAttr)(data)) {
        attrStr += joinDomAttr(key, (0, _utils.escape)(data));
      }
    }
  }

  return attrStr;
};

var hChildren = function hChildren(children) {
  var stack = children.reverse();
  var html = '';

  while (stack.length) {
    var child = stack.pop(); // 经过h函数生成的html

    if (child && child.hasOwnProperty('html')) {
      html += child.html;
      continue;
    } // string


    if (typeof child === 'string') {
      // child有可能为用户自定义的纯string，如<li>{nameStr}</li>中的nameStr,需要进行转义
      html += (0, _utils.escape)(child);
      continue;
    } // array


    if (child && child.push) {
      for (var i = child.length; i--;) {
        stack.push(child[i]);
      }

      continue;
    } // number


    if (typeof child === 'number') {
      html += child + '';
      continue;
    }
  }

  return html;
};
/**
 *  @return string | object
 * */


var h = function h(type, attrs) {
  for (var _len = arguments.length, children = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  attrs = attrs || {}; // dom element

  if (typeof type === 'string') {
    var html = '<' + type + getStaticMarkupAttrStr(attrs);

    if (!_consts.emptyTags[type]) {
      html += '>';

      if (attrs.hasOwnProperty('dangerouslySetInnerHTML')) {
        html += attrs.dangerouslySetInnerHTML.__html;
      }

      html += hChildren(children);
      html += '</' + type + '>';
    } else {
      html += '/>';
    }

    return {
      html: html,
      props: attrs,
      children: children
    };
  } // React.Fragment


  if (type === _react.default.Fragment) {
    return {
      html: hChildren(children)
    };
  } // class component


  if ((0, _utils.isReactComponent)(type)) {
    var props = _objectSpread({}, type.defaultProps || {}, attrs, {
      children: children
    });

    var instance = new type(props);
    instance.props = props;
    instance.componentWillMount && instance.componentWillMount();
    instance.UNSAFE_componentWillMount && instance.UNSAFE_componentWillMount();
    return instance.render();
  } // function component


  if (typeof type === 'function') {
    var _props = _objectSpread({}, type.defaultProps || {}, attrs, {
      children: children
    });

    return type(_props);
  }
};

exports.h = h;

function fastRenderToStaticMarkup(renderComponent) {
  // mock react interface runtime
  reactRuntime.mock();
  var html = renderComponent(); // recovery react interface runtime

  reactRuntime.recovery();

  if (!html) {
    return '';
  } else {
    return html.hasOwnProperty('html') ? html.html : html;
  }
}